# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/dreamai_pdf',
                'doc_host': 'https://HamzaFarhan.github.io',
                'git_url': 'https://github.com/HamzaFarhan/dreamai_pdf',
                'lib_path': 'dreamai_pdf'},
  'syms': { 'dreamai_pdf.core': { 'dreamai_pdf.core.cid_to_char': ('core.html#cid_to_char', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.extract_text': ('core.html#extract_text', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.process_text': ('core.html#process_text', 'dreamai_pdf/core.py')},
            'dreamai_pdf.imports': {},
            'dreamai_pdf.segment': { 'dreamai_pdf.segment.load_ems_model': ('segment.html#load_ems_model', 'dreamai_pdf/segment.py'),
                                     'dreamai_pdf.segment.load_segs_model': ('segment.html#load_segs_model', 'dreamai_pdf/segment.py'),
                                     'dreamai_pdf.segment.print_segments': ('segment.html#print_segments', 'dreamai_pdf/segment.py'),
                                     'dreamai_pdf.segment.text_to_segments': ('segment.html#text_to_segments', 'dreamai_pdf/segment.py'),
                                     'dreamai_pdf.segment.write_segments': ('segment.html#write_segments', 'dreamai_pdf/segment.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_segment.ipynb.

# %% auto 0
__all__ = ['text_to_segments', 'print_segments', 'load_segs_model', 'load_ems_model', 'write_segments']

# %% ../nbs/01_segment.ipynb 2
from .imports import *
from .core import *

# %% ../nbs/01_segment.ipynb 3
def text_to_segments(text: Union[str, List[str]], segs_model: SetFitModel,
                     thresh: float=0.6, classes: List=['Work Experience', 'Education', 'Certifications', 'Other']):
    preds = segs_model.predict(text).detach().cpu().numpy()
    probs = segs_model.predict_proba(text).detach().cpu().numpy().max(1)
    preds[probs < thresh] = 3
    pred_classes = [classes[p] for p in preds]
    segments = {pc:[] for pc in classes}
    for i,pc in enumerate(pred_classes):
        txt = text[i]
        if txt not in segments[pc]:
            segments[pc].append(text[i])
    return segments, pred_classes, probs

def print_segments(segments: dict, limit: int=10):
    for k,v in segments.items():
        print(f'{k}: {len(v)}')
        for s in v[:limit]:
            print('\t', end='')
            pprint(s, width=100)
            print()
        print('-'*100)
        
def load_segs_model(model_name: str='HamzaFarhan/PDFSegs', device: Union[str, torch.device]='cpu'):
    return SetFitModel.from_pretrained(model_name).to(device)

def load_ems_model(model_name: str='HamzaFarhan/PDFSegs', device: Union[str, torch.device]='cpu'):
    return SentenceTransformer(model_name, device=device)

def write_segments(segs_model: SetFitModel, data_path: Union[str, Path, List[Union[str, Path]]], output_path: Union[str, Path]='pdf_segments'):
    """
    Extracts text from PDFs and writes segments to JSON files.
    
    Parameters
    ----------
    segs_model : SetFitModel
        SetFit model for segment classification.
    data_path : Union[str, Path, List[Union[str, Path]]]
        Path to PDFs. Can be a single file, a directory, or a list of files/directories.
    output_path : Union[str, Path]
        Folder to write JSON files. Defaults to 'pdf_segments'.
    
    Returns
    -------
    None
    """
    output_path = Path(output_path)
    os.makedirs(output_path, exist_ok=True)
    pdfs = resolve_data_path(data_path)
    for file in pdfs:
        try:
            pdf_text = extract_text(file)[str(file)]
            segments, _, _ = text_to_segments(pdf_text, segs_model, thresh=0.6)
            fn = (output_path/Path(file).stem).with_suffix('.json')
            with open(fn, 'w') as f:
                json.dump(segments, f, indent=4)
        except Exception as e:
            print(f'\nCould not write segments for file: {str(file)}\n{e}')



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['cid_to_char', 'process_text', 'extract_text']

# %% ../nbs/00_core.ipynb 2
from .imports import *

# %% ../nbs/00_core.ipynb 3
def cid_to_char(cidx: str):
    try:
        return chr(int(re.findall(r'\(cid\:(\d+)\)',cidx)[0]) + 29)
    except:
        return cidx

def process_text(text: str):
    text = text.strip()
    text = cid_to_char(text)
    text = re.sub(u'\xa0', u' ', text)
    text = re.sub(" +", " ", text)
    text = re.sub(r"\uf0b7", " ", text)
    text = re.sub(r"\(cid:\d{0,3}\)", " ", text)
    text = re.sub(r'•', " ", text)
    text = re.sub(r'●', " ", text)
    text = re.sub(r'▪', " ", text)
    text = re.sub(r'➢', " ", text)
    text = re.sub(r'\u2013', " ", text)
    return text.strip()

def extract_text(data_path: Union[str, Path, List[Union[str, Path]]]):
    pdfs = resolve_data_path(data_path)
    pdf_dict = {}
    for file in pdfs:
        if Path(file).suffix == '.pdf':
            pdf = PdfReader(str(file))
            pdf_text = []
            for page in pdf.pages:
                text = page.extract_text().splitlines()
                for txt in text:
                    txt = process_text(txt)
                    if not txt.isspace() and len(txt) > 2:
                        if len(txt) > 512:
                            nw = 10
                            # split txt into smaller chunks of size 20 words
                            txt = txt.split()
                            txt = [' '.join(txt[i:i+nw]) for i in range(0, len(txt), nw)]
                        pdf_text.append(txt)
                        
                        # if len(page_text) == 0:
                        #     page_text.append(txt)
                        # else:
                        #     txt2 = page_text[-1]
                        #     if txt2[-1] not in ['.'] and len(txt2+txt) < 256:
                        #         page_text[-1] = txt2 + ' ' + txt
                        #     else:
                        #         page_text.append(txt)
                # pdf_text += page_text
            pdf_dict[str(file)] = flatten_list(pdf_text)
    return pdf_dict
